about summary refs log tree commit diff
diff options
context:
space:
mode:
authorBen Kimock <kimockb@gmail.com>2023-11-26 20:45:56 -0500
committerBen Kimock <kimockb@gmail.com>2023-11-28 18:03:50 -0500
commit5ef5e557495e55833b1c1a12dadab3a673d00b2a (patch)
treebc31f361c74c4c08f4d094de842b281c0a04965d
parent59f5d518527a6584062e278179f4b2e31f9243ff (diff)
downloadrust-5ef5e557495e55833b1c1a12dadab3a673d00b2a.tar.gz
rust-5ef5e557495e55833b1c1a12dadab3a673d00b2a.zip
Use a u64 for the rmeta root position
-rw-r--r--crates/proc-macro-api/src/version.rs21
1 files changed, 13 insertions, 8 deletions
diff --git a/crates/proc-macro-api/src/version.rs b/crates/proc-macro-api/src/version.rs
index 48efbf589c6..238fb561554 100644
--- a/crates/proc-macro-api/src/version.rs
+++ b/crates/proc-macro-api/src/version.rs
@@ -120,13 +120,18 @@ pub fn read_version(dylib_path: &AbsPath) -> io::Result<String> {
     let version = u32::from_be_bytes([dot_rustc[4], dot_rustc[5], dot_rustc[6], dot_rustc[7]]);
     // Last supported version is:
     // https://github.com/rust-lang/rust/commit/0696e79f2740ad89309269b460579e548a5cd632
-    let snappy_portion = match version {
-        5 | 6 => &dot_rustc[8..],
+    let (snappy_portion, bytes_before_version) = match version {
+        5 | 6 => (&dot_rustc[8..], 13),
         7 | 8 => {
             let len_bytes = &dot_rustc[8..12];
             let data_len = u32::from_be_bytes(len_bytes.try_into().unwrap()) as usize;
-            &dot_rustc[12..data_len + 12]
+            (&dot_rustc[12..data_len + 12], 13)
         }
+        9 => {
+            let len_bytes = &dot_rustc[8..16];
+            let data_len = u64::from_le_bytes(len_bytes.try_into().unwrap()) as usize;
+            (&dot_rustc[16..data_len + 12], 17)
+         }
         _ => {
             return Err(io::Error::new(
                 io::ErrorKind::InvalidData,
@@ -142,15 +147,15 @@ pub fn read_version(dylib_path: &AbsPath) -> io::Result<String> {
         Box::new(SnapDecoder::new(snappy_portion))
     };
 
-    // the bytes before version string bytes, so this basically is:
+    // We're going to skip over the bytes before the version string, so basically:
     // 8 bytes for [b'r',b'u',b's',b't',0,0,0,5]
-    // 4 bytes for [crate root bytes]
+    // 4 or 8 bytes for [crate root bytes]
     // 1 byte for length of version string
     // so 13 bytes in total, and we should check the 13th byte
     // to know the length
-    let mut bytes_before_version = [0u8; 13];
+    let mut bytes_before_version = vec![0u8; bytes_before_version];
     uncompressed.read_exact(&mut bytes_before_version)?;
-    let length = bytes_before_version[12];
+    let length = *bytes_before_version.last().unwrap();
 
     let mut version_string_utf8 = vec![0u8; length as usize];
     uncompressed.read_exact(&mut version_string_utf8)?;
@@ -142,15 +147,15 @@ pub fn read_version(dylib_path: &AbsPath) -> io::Result<String> {
     // 1 byte for length of version string
-    // so 13 bytes in total, and we should check the 13th byte
+    // so 13 or 17 bytes in total, and we should check the last of those bytes
     // to know the length